@@ -1,3 +1,11 @@
+import warnings
+
+warnings.warn(
+    "Django functionality is deprecated and will be removed in a later version (unless support is requested).",
+    DeprecationWarning,
+    stacklevel=2,
+)
+
 import operator
 from optimade.filterparser import LarkParser
 from lark import Tree
@@ -5,6 +13,9 @@ from lark.lexer import Token
 from django.db.models import Q
 
 
+__all__ = ("DjangoTransformer",)
+
+
 class DjangoQueryError(Exception):
     pass
 
@@ -19,7 +30,16 @@ django_db_keys = {
 }
 
 
-class Lark2Django:
+class DjangoTransformer:
+    """Filter transformer for implementations using Django.
+
+    !!! warning "Warning"
+        This transformer is deprecated as it only supports
+        the 0.9.7 grammar version, and works different to other
+        filter transformers in this package.
+
+    """
+
     def __init__(self):
         self.opers = {
             "=": self.eq,
