@@ -109,9 +109,11 @@ class ElasticTransformer(BaseTransformer):
             return Q(query_type, **{field: value})
 
         if op == "!=":
-            return ~Q(  # pylint: disable=invalid-unary-operand-type
-                query_type, **{field: value}
-            )
+            # != queries must also include an existence check
+            # Note that for MongoDB, `$exists` will include null-valued fields,
+            # where as in ES `exists` excludes them.
+            # pylint: disable=invalid-unary-operand-type
+            return ~Q(query_type, **{field: value}) & Q("exists", field=field)
 
     def _has_query_op(self, quantities, op, predicate_zip_list):
         """
@@ -216,7 +218,7 @@ class ElasticTransformer(BaseTransformer):
         )
 
     def __default__(self, tree, children, *args, **kwargs):
-        """ Default behavior for rules that only replace one symbol with another """
+        """Default behavior for rules that only replace one symbol with another"""
         return children[0]
 
     def filter(self, args):
