@@ -33,7 +33,7 @@ class MongoTransformer(BaseTransformer):
     }
 
     def postprocess(self, query):
-        """ Used to post-process the final parsed query. """
+        """Used to post-process the final parsed query."""
         if self.mapper:
             # important to apply length alias before normal aliases
             query = self._apply_length_aliases(query)
@@ -84,6 +84,12 @@ class MongoTransformer(BaseTransformer):
     def property_first_comparison(self, arg):
         # property_first_comparison: property ( value_op_rhs | known_op_rhs | fuzzy_string_op_rhs | set_op_rhs |
         # set_zip_op_rhs | length_op_rhs )
+
+        # Awkwardly, MongoDB will match null fields in $ne filters,
+        # so we need to add a check for null equality in evey $ne query.
+        if "$ne" in arg[1]:
+            return {"$and": [{arg[0]: arg[1]}, {arg[0]: {"$ne": None}}]}
+
         return {arg[0]: arg[1]}
 
     def constant_first_comparison(self, arg):
@@ -386,7 +392,7 @@ class MongoTransformer(BaseTransformer):
         """
 
         def check_for_id_key(prop, _):
-            """ Find cases where the query dict is operating on the `_id` field. """
+            """Find cases where the query dict is operating on the `_id` field."""
             return prop == "_id"
 
         def replace_str_id_with_objectid(subdict, prop, expr):
@@ -415,13 +421,13 @@ class MongoTransformer(BaseTransformer):
         """
 
         def check_for_timestamp_field(prop, _):
-            """ Find cases where the query dict is operating on a timestamp field. """
+            """Find cases where the query dict is operating on a timestamp field."""
             if self.mapper is not None:
                 prop = self.mapper.get_optimade_field(prop)
             return prop == "last_modified"
 
         def replace_str_date_with_datetime(subdict, prop, expr):
-            """Encode suspected dates in with BSON. """
+            """Encode suspected dates in with BSON."""
             import bson.json_util
 
             for operator in subdict[prop]:
