@@ -161,7 +161,7 @@ class ImplementationValidator:
         self.results = ValidatorResults(verbosity=self.verbosity)
 
     def _setup_log(self):
-        """ Define stdout log based on given verbosity. """
+        """Define stdout log based on given verbosity."""
         self._log = logging.getLogger("optimade").getChild("validator")
         self._log.handlers = []
         stdout_handler = logging.StreamHandler(sys.stdout)
@@ -184,7 +184,7 @@ class ImplementationValidator:
             self._log.setLevel(logging.DEBUG)
 
     def print_summary(self):
-        """ Print a summary of the results of validation. """
+        """Print a summary of the results of validation."""
         if self.respond_json:
             print(json.dumps(dataclasses.asdict(self.results), indent=2))
             return
@@ -827,6 +827,15 @@ class ImplementationValidator:
                             "returning different results each time."
                         )
 
+                # check that the filter returned no entries that had a null or missing value for the filtered property
+                if any(
+                    entry["attributes"].get(prop, entry.get(prop, None)) is None
+                    for entry in reversed_response.get("data", [])
+                ):
+                    raise ResponseError(
+                        f"Filter {reversed_query!r} on field {prop!r} returned entries that had null or missing values for the field."
+                    )
+
             excluded = operator in exclusive_operators
             # if we have all results on this page, check that the blessed ID is in the response
             if not response["meta"]["more_data_available"]:
@@ -848,6 +857,15 @@ class ImplementationValidator:
                         f"(with field '{prop} = {test_value}') potentially indicating a problem with filtering on this field."
                     )
 
+            # check that the filter returned no entries that had a null or missing value for the filtered property
+            if any(
+                entry["attributes"].get(prop, entry.get(prop, None)) is None
+                for entry in response.get("data", [])
+            ):
+                raise ResponseError(
+                    f"Filter {query!r} on field {prop!r} returned entries that had null or missing values for the field."
+                )
+
         if prop in CONF.unique_properties:
             if num_data_returned["="] is not None and num_data_returned["="] == 0:
                 raise ResponseError(
@@ -1327,6 +1345,14 @@ class ImplementationValidator:
                 raise ResponseError(
                     f'Illegal entry "{non_entry_endpoint}" was found in entry_types_by_format"'
                 )
+
+        # Filter out custom extension endpoints that are not covered in the specification
+        available_json_entry_endpoints = [
+            endp
+            for endp in available_json_entry_endpoints
+            if endp in CONF.entry_endpoints
+        ]
+
         return (
             available_json_entry_endpoints,
             "successfully found available entry types in baseinfo",
